@@ -30,11 +30,8 @@
 #include "textinput_v3_interface_p.h"
 #include "touch_interface_p.h"
 #include "utils.h"
-// linux
-#include <config-kwaylandserver.h>
-#if HAVE_LINUX_INPUT_H
+
 #include <linux/input.h>
-#endif
 
 #include <functional>
 
@@ -631,7 +628,6 @@ PointerInterface *SeatInterface::pointer() const
 
 static quint32 qtToWaylandButton(Qt::MouseButton button)
 {
-#if HAVE_LINUX_INPUT_H
     static const QHash<Qt::MouseButton, quint32> s_buttons({
         {Qt::LeftButton, BTN_LEFT},
         {Qt::RightButton, BTN_RIGHT},
@@ -652,10 +648,6 @@ static quint32 qtToWaylandButton(Qt::MouseButton button)
         // further mapping not possible, 0x120 is BTN_JOYSTICK
     });
     return s_buttons.value(button, 0);
-#else
-    Q_UNUSED(button)
-    return 0;
-#endif
 }
 
 bool SeatInterface::isPointerButtonPressed(Qt::MouseButton button) const
@@ -854,6 +846,42 @@ void SeatInterface::cancelPointerPinchGesture()
     }
 }
 
+void SeatInterface::startPointerHoldGesture(quint32 fingerCount)
+{
+    if (!d->pointer) {
+        return;
+    }
+
+    auto holdGesture = PointerHoldGestureV1Interface::get(pointer());
+    if (holdGesture) {
+        holdGesture->sendBegin(d->display->nextSerial(), fingerCount);
+    }
+}
+
+void SeatInterface::endPointerHoldGesture()
+{
+    if (!d->pointer) {
+        return;
+    }
+
+    auto holdGesture = PointerHoldGestureV1Interface::get(pointer());
+    if (holdGesture) {
+        holdGesture->sendEnd(d->display->nextSerial());
+    }
+}
+
+void SeatInterface::cancelPointerHoldGesture()
+{
+    if (!d->pointer) {
+        return;
+    }
+
+    auto holdGesture = PointerHoldGestureV1Interface::get(pointer());
+    if (holdGesture) {
+        holdGesture->sendCancel(d->display->nextSerial());
+    }
+}
+
 SurfaceInterface *SeatInterface::focusedKeyboardSurface() const
 {
     return d->globalKeyboard.focus.surface;
@@ -969,6 +997,11 @@ TouchInterface *SeatInterface::touch() const
     return d->touch.data();
 }
 
+QPointF SeatInterface::firstTouchPointPosition() const
+{
+    return d->globalTouch.focus.firstTouchPos;
+}
+
 void SeatInterface::setFocusedTouchSurface(SurfaceInterface *surface, const QPointF &surfacePosition)
 {
     if (!d->touch) {
@@ -986,7 +1019,8 @@ void SeatInterface::setFocusedTouchSurface(SurfaceInterface *surface, const QPoi
     }
     d->globalTouch.focus = SeatInterfacePrivate::Touch::Focus();
     d->globalTouch.focus.surface = surface;
-    d->globalTouch.focus.offset = surfacePosition;
+    setFocusedTouchSurfacePosition(surfacePosition);
+
     if (d->globalTouch.focus.surface) {
         d->globalTouch.focus.destroyConnection = connect(surface, &QObject::destroyed, this, [this]() {
             if (isTouchSequence()) {
@@ -1002,6 +1036,8 @@ void SeatInterface::setFocusedTouchSurface(SurfaceInterface *surface, const QPoi
 void SeatInterface::setFocusedTouchSurfacePosition(const QPointF &surfacePosition)
 {
     d->globalTouch.focus.offset = surfacePosition;
+    d->globalTouch.focus.transformation = QMatrix4x4();
+    d->globalTouch.focus.transformation.translate(-surfacePosition.x(), -surfacePosition.y());
 }
 
 void SeatInterface::notifyTouchDown(qint32 id, const QPointF &globalPosition)
@@ -1017,7 +1053,6 @@ void SeatInterface::notifyTouchDown(qint32 id, const QPointF &globalPosition)
         d->globalTouch.focus.firstTouchPos = globalPosition;
     }
 
-#if HAVE_LINUX_INPUT_H
     if (id == 0 && hasPointer() && focusedTouchSurface()) {
         TouchInterfacePrivate *touchPrivate = TouchInterfacePrivate::get(d->touch.data());
         if (touchPrivate->touchesForClient(focusedTouchSurface()->client()).isEmpty()) {
@@ -1028,7 +1063,6 @@ void SeatInterface::notifyTouchDown(qint32 id, const QPointF &globalPosition)
             d->pointer->sendFrame();
         }
     }
-#endif
 
     d->globalTouch.ids[id] = serial;
 }
@@ -1075,7 +1109,6 @@ void SeatInterface::notifyTouchUp(qint32 id)
     }
     d->touch->sendUp(id, serial);
 
-#if HAVE_LINUX_INPUT_H
     if (id == 0 && hasPointer() && focusedTouchSurface()) {
         TouchInterfacePrivate *touchPrivate = TouchInterfacePrivate::get(d->touch.data());
         if (touchPrivate->touchesForClient(focusedTouchSurface()->client()).isEmpty()) {
@@ -1085,7 +1118,6 @@ void SeatInterface::notifyTouchUp(qint32 id)
             d->pointer->sendFrame();
         }
     }
-#endif
 
     d->globalTouch.ids.remove(id);
 }
@@ -1152,26 +1184,25 @@ void SeatInterface::setFocusedTextInputSurface(SurfaceInterface *surface)
 {
     const quint32 serial = d->display->nextSerial();
 
+    if (d->focusedTextInputSurface == surface) {
+        return;
+    }
+
     if (d->focusedTextInputSurface) {
         disconnect(d->focusedSurfaceDestroyConnection);
-    }
-
-    if (d->focusedTextInputSurface != surface) {
         d->textInputV2->d->sendLeave(serial, d->focusedTextInputSurface);
         d->textInputV3->d->sendLeave(d->focusedTextInputSurface);
-        d->focusedTextInputSurface = surface;
-        Q_EMIT focusedTextInputSurfaceChanged();
     }
+    d->focusedTextInputSurface = surface;
+    Q_EMIT focusedTextInputSurfaceChanged();
 
-    if (d->focusedTextInputSurface) {
+    if (surface) {
         d->focusedSurfaceDestroyConnection = connect(surface, &SurfaceInterface::aboutToBeDestroyed, this, [this] {
             setFocusedTextInputSurface(nullptr);
         });
+        d->textInputV2->d->sendEnter(surface, serial);
+        d->textInputV3->d->sendEnter(surface);
     }
-
-    d->textInputV2->d->sendEnter(surface, serial);
-    d->textInputV3->d->sendEnter(surface);
-    // TODO: setFocusedSurface like in other interfaces
 }
 
 SurfaceInterface *SeatInterface::focusedTextInputSurface() const
@@ -1281,7 +1312,7 @@ void SeatInterface::startDrag(AbstractDataSource *dragSource, SurfaceInterface *
         d->drag.transformation = d->globalPointer.focus.transformation;
     } else if (hasImplicitTouchGrab(dragSerial)) {
         d->drag.mode = SeatInterfacePrivate::Drag::Mode::Touch;
-        // TODO: touch transformation
+        d->drag.transformation = d->globalTouch.focus.transformation;
     } else {
         // no implicit grab, abort drag
         return;
@@ -1290,8 +1321,6 @@ void SeatInterface::startDrag(AbstractDataSource *dragSource, SurfaceInterface *
 
     // set initial drag target to ourself
     d->drag.surface = originSurface;
-    // TODO: transformation needs to be either pointer or touch
-    d->drag.transformation = d->globalPointer.focus.transformation;
 
     d->drag.source = dragSource;
     if (dragSource) {
